/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: GenericDocBuilderGenerator.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.xmlc.dom.generic;

import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.codegen.JavaClass;
import org.enhydra.xml.xmlc.codegen.JavaCode;
import org.enhydra.xml.xmlc.codegen.JavaLang;
import org.enhydra.xml.xmlc.codegen.JavaMethod;
import org.enhydra.xml.xmlc.compiler.ElementTable;
import org.enhydra.xml.xmlc.dom.AccessorGenerator;
import org.enhydra.xml.xmlc.dom.DocBuilderGenerator;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.w3c.dom.Document;

/**
 * Class to generate code to build the document tree using only the W3C DOM
 * interface.  The document building code is generated in several functions to
 * avoid the limit on the maximum size of a method.
 */
abstract public class GenericDocBuilderGenerator implements DocBuilderGenerator {
    /*
     * Maximum creation-cost per build method.  One unit is approximately the
     * cost to create one node.
     */
    private static final int MAX_CREATE_COST_PER_BUILD_METHOD = 512;

    /*
     * Name of variable in init function containing Document.
     */
    private static final String DOCUMENT_VAR_NAME = "document";

    /**
     * Base name of DOM building methods.
     */
    private static final String SUBDOCUMENT_BUILD_METHOD = "buildSubDocument";

    /**
     * Generate the document builder method.
     * @see DocBuilderGenerator#createBuildDocumentMethod
     */
    public void createBuildDocumentMethod(XMLCDocument xmlcDoc,
                                          AccessorGenerator accessorGenerator,
                                          ElementTable elementTable,
                                          JavaClass docClass,
                                          JavaMethod buildDocumentMethod) throws XMLCException {
        // Create code to create DOM.
        DOMBuilderGenerator domBuilderGenerator
            = new DOMBuilderGenerator(SUBDOCUMENT_BUILD_METHOD,
                                      xmlcDoc.getDocument(),
                                      Document.class.getName(),
                                      new NodeCreateGenerator(xmlcDoc),
                                      accessorGenerator,
                                      elementTable,
                                      docClass,
                                      MAX_CREATE_COST_PER_BUILD_METHOD,
                                      false);
        // Generate the document building method.
        JavaCode body = buildDocumentMethod.getCode();
        body.add(xmlcDoc.getDocument().getClass().getName() + " "
                 + DOCUMENT_VAR_NAME + " = ");
        domBuilderGenerator.createMethodCall(body);

        // Save document information.
        body.addln("setDocument(" + DOCUMENT_VAR_NAME + ","
                   + JavaLang.createStringConst(xmlcDoc.getDomFactory().getMIMEType()) + ", "
                   + JavaLang.createStringConst(xmlcDoc.getEncoding()) + ");");
        body.addln();
    }
}
